import os
import sys
import json
import pymongo
from pymongo import MongoClient, InsertOne
from decimal import Decimal
from bson.decimal128 import Decimal128
from bson.codec_options import TypeCodec, TypeRegistry, CodecOptions

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

DOCS_PER_WRITE = 1000

collection		= sys.argv[1]
jsonl_path 	= sys.argv[2]
mongo_schema 	= sys.argv[3]

if len(sys.argv) > 4 and sys.argv[4].upper() == 'TRUE':
	do_drop_collection = True
else:
	do_drop_collection = False

if len(sys.argv) > 5:
	doc_key = sys.argv[5]
else:
	doc_key = None

gxd_dir 		= os.environ['GXD_PDK_DIR']
custom_conf 	= os.environ['GXD_PDK_CONFIG']
ingest_id 		= os.environ['GXD_INGEST_JOB_ID']

fields_tsv 	= os.path.join( gxd_dir, "config", gxd.get_config_setting('FIELD_CONFIGS', gxd_dir, custom_conf) )

mongo_host 	= gxd.get_config_setting('MONGO_HOST', gxd_dir, custom_conf)
mongo_port 	= gxd.get_config_setting('MONGO_PORT', gxd_dir, custom_conf)
mongo_user 	= gxd.get_config_setting('MONGO_USER', gxd_dir, custom_conf)
mongo_pass 	= gxd.get_config_setting('MONGO_PASS', gxd_dir, custom_conf)

fields 		= gxd.get_fields(fields_tsv, collection)

injections = []

# Use Python function attribute to create auto-counter that increments with every function call
def counter():
	if not hasattr(counter,'value'):
		counter.value = 0
	counter.value += 1
	return counter.value

row_ordinal_field_name = gxd.get_pdk_field_name(fields, 'BATCH_UPDATE_ORD')
if row_ordinal_field_name:
	injections.append( { 'key': row_ordinal_field_name, 'function': counter } )

ingest_id_field_name = gxd.get_pdk_field_name(fields, 'INGEST_ID')
if ingest_id_field_name:
	injections.append( { 'key': ingest_id_field_name, 'function': lambda : ingest_id } )


try:
	db = MongoClient( "{}:{}".format(mongo_host, mongo_port), username=mongo_user, password=mongo_pass )[mongo_schema]
except pymongo.errors.PyMongoError as e:
	gxd.error("\t{0}: exception connecting to Mongo database: {1}\n".format( os.path.basename(__file__), str(e) ))
	sys.exit(1)


class DecimalCodec(TypeCodec):

	python_type = Decimal
	bson_type = Decimal128

	def transform_python(self, value):
		return Decimal128(value)

	def transform_bson(self, value):
		return value.to_decimal()

decimal_codec = DecimalCodec()
type_registry = TypeRegistry([decimal_codec])
codec_options = CodecOptions(type_registry=type_registry)
coll = db.get_collection(collection, codec_options=codec_options)

if do_drop_collection:
	coll.drop()

jsonl = gxd.jsonl_iterator( os.path.join(jsonl_path, collection + ".jsonl"), injections = injections )
write_cnt 	= 0

for group in gxd.group_iterator(jsonl, DOCS_PER_WRITE):
	operations = []
	for document in group:
		if doc_key:
			document['_id'] = document.get(doc_key)
		operations.append( InsertOne(document) )
	write_cnt += 1
	try:
		coll.bulk_write( operations, ordered=False )
		sys.stdout.write("\r\tDatabase writes ({} documents each): {} ".format( DOCS_PER_WRITE, write_cnt))
	except pymongo.errors.PyMongoError as e:
		gxd.error("\t{0}: exception writing to Mongo database: {1}\n".format( os.path.basename(__file__), str(e) ))
		sys.exit(1)

sys.stdout.write("\n")
gxd.log("\tDatabase writes ({0} documents each): {1}\n".format( DOCS_PER_WRITE, write_cnt))

