#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Core replace sequence for Mongo
#  =============================================================================

. $GXD_PDK_DIR/bin/functions.sh
load_config

VARS=$GXD_PDK_DIR/vars
echo "JOB_PHASE=CORE_REPLACE" > $VARS/job_phase.var

echo

# Check which step the script failed on during previous run, if any
# If it is not 0, we'll skip to the failed step
JOB_STATUS=$1
if  [ $JOB_STATUS = "ERROR" ] &&
	[ -e $VARS/job_phase_step.var ]
then
	. $VARS/job_phase_step.var
else
	JOB_PHASE_STEP=0
fi

# 1. Run replace sequence for each entity/collection
if [ $JOB_PHASE_STEP -le 1 ]; then
	echo "JOB_PHASE_STEP=1" > $VARS/job_phase_step.var

	# Since we're advancing collection by collection, ensure that tracker is created (or exists) to record successfully processed collections
	touch $VARS/job_phase_step_tracker.var

	while read -r ENTITY_CONFIG; do
		ENTITY=$( echo "$ENTITY_CONFIG" | cut -f1 )

		# Only proceed if the collection has not already been successfully processed
		FOUND=$( grep -w $ENTITY $VARS/job_phase_step_tracker.var )
		if [ "$FOUND" = "" ]; then

			msg_ok "REPLACE: $PYTHON db_mongo/core.py $SCHEMA_PREFIX $ENTITY REPLACE"
			$PYTHON $GXD_PDK_DIR/bin/db_mongo/core.py $SCHEMA_PREFIX $ENTITY REPLACE #TRUE
			if [ $? != 0 ]; then
				msg_err "REPLACE: ERROR: $PYTHON db_mongo/core.py $SCHEMA_PREFIX $ENTITY REPLACE returned error"
				exit_abnormal
			fi

			# Record collection as successfully processed
			echo "$ENTITY" >> $VARS/job_phase_step_tracker.var
		else
			msg_ok "REPLACE: Collection $ENTITY has already been processed"
		fi

	done < $GXD_PDK_DIR/config/$ENTITY_CONFIGS

	# Remove tracker if all collections have been successfully processed
	rm -f $VARS/job_phase_step_tracker.var

	echo
fi


# Once all steps ran successfully, clear previously failed step (if any)
rm -f $VARS/job_phase_step.var

echo
