#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Core replace sequence for Postgres
#  =============================================================================

. $GXD_PDK_DIR/bin/functions.sh
load_config

VARS=$GXD_PDK_DIR/vars
GXD_TSV_DIR=$GXD_PDK_DIR/files/tsv
echo "JOB_PHASE=CORE_REPLACE" > $VARS/job_phase.var

SCHEMA_CORE=${SCHEMA_PREFIX}_core
SCHEMA_STAGE=${SCHEMA_PREFIX}_stage

echo

# Check which step the script failed on during previous run, if any
# If it is not 0, we'll skip to the failed step
JOB_STATUS=$1
if  [ $JOB_STATUS = "ERROR" ] &&
	[ -e $VARS/job_phase_step.var ]
then
	. $VARS/job_phase_step.var
else
	JOB_PHASE_STEP=0
fi

while read LINE; do
	PDK_VER=$LINE
done < $GXD_PDK_DIR/docs/version.txt

if [ $BOOL_TO_INT = "TRUE" ]; then
	TRUTH_VALUE="1"
else
	TRUTH_VALUE="TRUE"
fi

# 1. Create schema
if [ $JOB_PHASE_STEP -le 1 ]; then
	echo "JOB_PHASE_STEP=1" > $VARS/job_phase_step.var

	if [ -e $VARS/last_good_ingest_id.var ]; then
		. $VARS/last_good_ingest_id.var
	fi

	# Check if schema exists first
	msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py \"SELECT 'TRUE' AS result FROM pg_namespace WHERE nspname = '${SCHEMA_CORE}';\""
	SCHEMA_EXISTS=$( echo "SELECT 'TRUE' AS result FROM pg_namespace WHERE nspname = '{0}';" | $PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py "" "$SCHEMA_CORE" FALSE ";" "FETCH_ONE" )
	if [ $? != 0 ]; then
		msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py \"SELECT 'TRUE' AS result FROM pg_namespace WHERE nspname = '${SCHEMA_CORE}';\" returned error"
		exit_abnormal
	fi

	if [ "$SCHEMA_EXISTS" != "TRUE" ]; then
		msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/schema_core.sql '$SCHEMA_CORE|$PGSQL_USER' FALSE ;"
		$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/schema_core.sql "$SCHEMA_CORE|$PGSQL_USER" FALSE ";"
		if [ $? != 0 ]; then
			msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/schema_core.sql '$SCHEMA_CORE|$PGSQL_USER' returned error"
			exit_abnormal
		fi	

		# Create _vars table and initialize variables (PDK version)
		msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/vars.sql '$SCHEMA_CORE|$PGSQL_USER|$PDK_VER'"
		$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/vars.sql "$SCHEMA_CORE|$PGSQL_USER|$PDK_VER"
		if [ $? != 0 ]; then
			msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/vars.sql '$SCHEMA_CORE|$PGSQL_USER|$PDK_VER' returned error"
			exit_abnormal
		fi	
	fi
fi

# 2. Drop core constraints
if [ $JOB_PHASE_STEP -le 2 ]; then
	echo "JOB_PHASE_STEP=2" > $VARS/job_phase_step.var
	msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/drop_constraints.sql $SCHEMA_CORE"
	$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/replace_core/drop_constraints.sql $SCHEMA_CORE
	if [ $? != 0 ]; then
		msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/drop_constraints.sql $SCHEMA_CORE returned error"
		exit_abnormal
	fi
fi

# 3. Drop core indexes
if [ $JOB_PHASE_STEP -le 3 ]; then
	echo "JOB_PHASE_STEP=3" > $VARS/job_phase_step.var
	msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/drop_indexes.sql $SCHEMA_CORE"
	$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/replace_core/drop_indexes.sql $SCHEMA_CORE
	if [ $? != 0 ]; then
		msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/drop_indexes.sql $SCHEMA_CORE returned error"
		exit_abnormal
	fi
fi

# 4. Drop stage indexes (we'll be truncating stage data after successful copy to core)
if [ $JOB_PHASE_STEP -le 4 ]; then
	echo "JOB_PHASE_STEP=4" > $VARS/job_phase_step.var
	msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/stage/drop_indexes.sql $SCHEMA_STAGE"
	$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/stage/drop_indexes.sql $SCHEMA_STAGE
	if [ $? != 0 ]; then
		msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/stage/drop_indexes.sql $SCHEMA_STAGE returned error"
		exit_abnormal
	fi
fi

# 5. Delete stage objects flagged for deletion
if [ $JOB_PHASE_STEP -le 5 ]; then
	echo "JOB_PHASE_STEP=5" > $VARS/job_phase_step.var
	if [ $DB_DELETES = "TRUE" ]; then
		msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/stage/remove_deleted.sql '$SCHEMA_STAGE|$TRUTH_VALUE'"
		$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/stage/remove_deleted.sql "$SCHEMA_STAGE|$TRUTH_VALUE"
		if [ $? != 0 ]; then
			msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/stage/remove_deleted.sql '$SCHEMA_STAGE|$TRUTH_VALUE' returned error"
			exit_abnormal
		fi
	fi
fi

# 6. Move stage data to core if exists in stage
if [ $JOB_PHASE_STEP -le 6 ]; then
	echo "JOB_PHASE_STEP=6" > $VARS/job_phase_step.var

	# Since we're advancing table by table, ensure that tracker is created (or exists) to record successfully processed tables
	touch $VARS/job_phase_step_tracker.var

	TABLES=$( cut -f2 $GXD_PDK_DIR/config/$FIELD_CONFIGS | sort | uniq )
	for TABLE in $TABLES; do

		# Only proceed if the table has not already been successfully processed
		FOUND=$( grep -w $TABLE $VARS/job_phase_step_tracker.var )
		if [ "$FOUND" = "" ]; then

			# Retrieve row count for the current table
			msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py \"SELECT COUNT(*) FROM $SCHEMA_STAGE.$TABLE;\""
			ROW_COUNT=$( echo "SELECT COUNT(*) FROM {0}.{1};" | $PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py "" "$SCHEMA_STAGE|$TABLE" FALSE ";" "FETCH_ONE" )
			if [ $? != 0 ]; then
				msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py \"SELECT COUNT(*) FROM $SCHEMA_STAGE.$TABLE;\" returned error"
				exit_abnormal
			fi
			msg_ok "REPLACE: -> $ROW_COUNT"

			# If no rows, simply truncate the core table
			if [ "$ROW_COUNT" = "0" ]; then
				msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py \"TRUNCATE TABLE $SCHEMA_CORE.$TABLE;\""
				echo "TRUNCATE TABLE {0}.{1};" | $PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py "" "$SCHEMA_CORE|$TABLE"
				if [ $? != 0 ]; then
					msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py \"TRUNCATE TABLE $SCHEMA_CORE.$TABLE;\" returned error"
					exit_abnormal
				fi
			else
				# Otherwise truncate the core table, copy data from stage, truncate stage table, in one transaction
				msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/move_stage_to_core.sql '$SCHEMA_STAGE|$SCHEMA_CORE|$TABLE'"
				$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/replace_core/move_stage_to_core.sql "$SCHEMA_STAGE|$SCHEMA_CORE|$TABLE" FALSE
				if [ $? != 0 ]; then
					msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/move_stage_to_core.sql '$SCHEMA_STAGE|$SCHEMA_CORE|$TABLE' returned error"
					exit_abnormal
				fi
			fi

			# Record table as successfully processed
			echo "$TABLE" >> $VARS/job_phase_step_tracker.var

		else
			msg_ok "REPLACE: Table $TABLE has already been processed"
		fi

	done

	# Remove tracker if all tables have been successfully processed
	rm -f $VARS/job_phase_step_tracker.var
fi

# 7. Recreate indexes in core
if [ $JOB_PHASE_STEP -le 7 ]; then
	echo "JOB_PHASE_STEP=7" > $VARS/job_phase_step.var
	msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/create_indexes.sql $SCHEMA_CORE"
	$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/replace_core/create_indexes.sql $SCHEMA_CORE TRUE
	if [ $? != 0 ]; then
		msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/create_indexes.sql $SCHEMA_CORE returned error"
		exit_abnormal
	fi
fi

# 8. Add core constraints; since Postgres does not support ADD CONSTRAINT IF EXISTS syntax, try to drop them first
if [ $JOB_PHASE_STEP -le 8 ]; then
	echo "JOB_PHASE_STEP=8" > $VARS/job_phase_step.var
	msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/drop_constraints.sql $SCHEMA_CORE"
	$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/replace_core/drop_constraints.sql $SCHEMA_CORE
	if [ $? != 0 ]; then
		msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/drop_constraints.sql $SCHEMA_CORE returned error"
		exit_abnormal
	fi
	msg_ok "REPLACE: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/add_constraints.sql $SCHEMA_CORE"
	$PYTHON $GXD_PDK_DIR/bin/db_pgsql/runsql.py $GXD_PDK_DIR/db/pgsql/replace_core/add_constraints.sql $SCHEMA_CORE TRUE
	if [ $? != 0 ]; then
		msg_err "REPLACE: ERROR: $PYTHON db_pgsql/runsql.py db/pgsql/replace_core/add_constraints.sql $SCHEMA_CORE returned error"
		exit_abnormal
	fi
fi


# Once all steps ran successfully, clear previously failed step
rm -f $VARS/job_phase_step.var

echo
