#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Iterates over entries in endpoint configuration file
#  For each endpoint, runs download_endpoint script
#  =============================================================================

. $GXD_PDK_DIR/bin/functions.sh
load_config

VARS=$GXD_PDK_DIR/vars
mkdir -p $GXD_PDK_DIR/files/xml
#echo "JOB_PHASE=DOWNLOAD" > $VARS/job_phase.var
echo "JOB_PHASE_STEP=API" > $VARS/job_phase_step.var

echo 

# Check if any of the API variables in the config file are zero/empty
if [ -z $BASE_URL ] || 
   [ -z $APIKEY ] || 
   [ -z $UPDATE_ID_LOCATION ] ||
   [ -z $ENDPOINT_CONFIGS ]
then
	msg_err "DOWNLOAD(API): ERROR: One or more variables: BASE_URL | APIKEY | ENDPOINT_CONFIGS | UPDATE_ID_LOCATION is not set in config"
	exit_abnormal
fi

# For every endpoint, extract limit and update_id paths from endpoint configuration file and call endpoint download script
while read -r ENDPOINT_CONFIG; do

	ENDPOINT=$( echo "$ENDPOINT_CONFIG" | cut -f1 )
	LIMIT=$( echo "$ENDPOINT_CONFIG" | cut -f2 )
	UPDATE_ID_XPATH=$( echo "$ENDPOINT_CONFIG" | cut -f3 )
	MAX_UPDATE_ID_XPATH=$( echo "$ENDPOINT_CONFIG" | cut -f4 )
	URL_PATH=$( echo "$ENDPOINT_CONFIG" | cut -f5 )

	if [ -z $URL_PATH ]; then
		URL_PATH=$ENDPOINT
	fi

	# Initialize update_id at zero and load any previously stored update_id value for the endpoint
	UPDATE_ID=0
	if [ -e $VARS/update_id_$ENDPOINT.var ]; then
	    . $VARS/update_id_$ENDPOINT.var
	fi

	msg_ok "DOWNLOAD(API): Starting with update_id $UPDATE_ID for $ENDPOINT endpoint"
	$GXD_PDK_DIR/bin/download_endpoint.sh $ENDPOINT $URL_PATH $UPDATE_ID $LIMIT $UPDATE_ID_XPATH "$MAX_UPDATE_ID_XPATH"
	if [ $? != 0 ]; then
		msg_err "DOWNLOAD(API): ERROR: download_endpoint.sh returned error on ${ENDPOINT}, ${UPDATE_ID_XPATH}, ${LIMIT}"
		exit_abnormal
	fi
	echo

	# Store endpoint XML names/dates/sizes for debug purposes
	find files/xml -name "$ENDPOINT-*.xml" -exec ls -log {} \; | sed 's/\/\//\//g' > $GXD_LOG_DIR/ls_xml_$ENDPOINT.log

done < $GXD_PDK_DIR/config/$ENDPOINT_CONFIGS
