#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Takes XML filename and an xpath for a numeric update_id value
#  Finds the maximum non-zero update_id value in the XML file at given xpath
#  Prints the maximum non-zero value or -1 if there is no such value
#  =============================================================================

import os
import sys
import xml.etree.cElementTree as ET

filename 		= sys.argv[1]
update_id_xpath = sys.argv[2]

if len(sys.argv) > 3:
	aggregate_function = sys.argv[3].upper()
else:
	aggregate_function = None

xpath_array = update_id_xpath.split('/')
xpath_len 	= len(xpath_array)
tag 		= xpath_array[-1]
tag_level 	= xpath_len-1

# drop xpath elements: first (to avoid dealing with root element namespaces) and last (which is the tag)
partial_xpath = '/'.join( xpath_array[ 1 : tag_level ] )

def num(s):
	try:
		return int(s)
	except ValueError:
		return float(s)

value = None
level = 0
xpath = []

for event, element in ET.iterparse(filename, events = ('end','start')):
	if event == 'start':
		level += 1
		xpath.append(element.tag)
	elif event == 'end':
		level -= 1
		xpath.pop()
		if level == tag_level and element.tag == tag and '/'.join( xpath[ 1 : tag_level ] ) == partial_xpath:
			text = element.text.strip()
			if aggregate_function:
				if value is None:
					if aggregate_function in ['COUNT','SUM']:
						value = 0
					elif aggregate_function in ['MAX','MIN']:
						value = num(text)
					else:
						pass
				if aggregate_function == 'COUNT':
					value += 1
				elif aggregate_function == 'SUM':
					value += num(text)
				elif aggregate_function == 'MAX':
					if num(text) > value:
						value = num(text)
				elif aggregate_function == 'MIN':
					if num(text) < value:
						value = num(text)
			else:
				print(text)
		element.clear()

if value is not None:
	print(str(value))
