#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Common shell functions used by various components of the product demo kit
#  =============================================================================


msg_err() {
	>&2 printf "$1\n"
	>>$GXD_LOG printf "$(date +%Y%m%d%H%M%S): $1\n"
	echo
}

msg_ok() {
	printf "$1\n"
	>>$GXD_LOG printf "$(date +%Y%m%d%H%M%S): $1\n"
}

log_ok() {
	>>$GXD_LOG printf "$(date +%Y%m%d%H%M%S): $1\n"
}

check_for() {
    command -v $1 >/dev/null 2>&1 || { 
        echo >&2 "ERROR: $1 required"; 
        exit 1; 
    }
}

load_config() {
	# Check if default config file exists and if so, load it
	if [ -e $GXD_PDK_DIR/config/default.conf ]; then
	    . $GXD_PDK_DIR/config/default.conf
	fi
	# Check if config file is supplied
	if [ -n "$GXD_PDK_CONFIG" ]; then
		# Check if config file exists and if so, load it; else error out
		if [ -e "$GXD_PDK_DIR/config/$GXD_PDK_CONFIG" ]; then
	    	. $GXD_PDK_DIR/config/$GXD_PDK_CONFIG
    	else
			msg_err "ERROR: Supplied config file $GXD_PDK_CONFIG does not exist in config/ directory"
			exit 1
		fi
	fi
}

exit_abnormal() {
	echo "JOB_STATUS=ERROR" > $VARS/job_status.var
	exit 1
}

exit_normal() {
	echo "JOB_STATUS=OK" > $VARS/job_status.var
	exit 0	
}

ctrl_C_handler() {
	exit_abnormal
}