#!/bin/sh

#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Receives a list of XML files to parse for a single resource (endoint/file)
#  Receives RESOURCE name and PARSER name
#  Calls parse_PARSER.py script for each XML file
#  Logs XML and alerts if any validation errors/warnings occur
#  =============================================================================

. $GXD_PDK_DIR/bin/functions.sh
load_config

FILELIST=$1
RESOURCE=$2
PARSER=$3

#TSV_DIR=files/tsv/$RESOURCE
FILES_DIR=files
mkdir -p $GXD_LOG_DIR/errors

# Use stdin if FILELIST is empty
if [ -z $FILELIST ]; then
	FILELIST=/dev/stdin
fi

# Default to RESOURCE name if PARSER name was not provided
if [ -z $PARSER ]; then
	PARSER=$RESOURCE
fi

while read -r FILENAME; do
	
	# Future: parallel parsing - check error flag and exit if error flag was set by a parallel parsing processes

	GXD_XML_FILE=$GXD_PDK_DIR/files/xml/$FILENAME
	GXD_VALIDATION_LOG=$GXD_LOG_DIR/errors/validation_$FILENAME.log

	# Parse single XML file and write validation output to log
	msg_ok "PARSE: $PYTHON bin/parsers/parse_$PARSER.py $FILENAME $FILES_DIR $RESOURCE"
	$PYTHON $GXD_PDK_DIR/bin/parsers/parse_$PARSER.py $GXD_XML_FILE $GXD_PDK_DIR/$FILES_DIR $RESOURCE 1>${GXD_VALIDATION_LOG}
	if [ $? != 0 ]; then
		# Future: parallel parsing - set error flag for parallel parsing processes
		msg_err "PARSE: ERROR: parse_$PARSER.py returned error parsing ${FILENAME}"
		cp $GXD_XML_FILE $GXD_LOG_DIR/errors/
		exit_abnormal
	fi

	# If validation log is not emtpy, notify of any data validation errors and gz/log offending XML
	if [ -s $GXD_VALIDATION_LOG ]; then
		msg_ok "PARSE: WARNING: Data validation errors/warnings encountered:"
		msg_ok "\t$GXD_VALIDATION_LOG"
		if [ $LOG_INVALID_SOURCE = "TRUE" ]; then
			gzip -f -k $GXD_XML_FILE
			mv $GXD_XML_FILE.gz $GXD_LOG_DIR/errors/
		fi
	else
		rm $GXD_VALIDATION_LOG
	fi

	# If successfully parsed, delete or gz/log XML file
	if [ $WITH_XML_AFTER_PARSE = "DELETE" ]; then
		rm -f $GXD_XML_FILE
	elif [ $WITH_XML_AFTER_PARSE = "LOG" ]; then
		mkdir -p $GXD_LOG_DIR/xml
		gzip -f $GXD_XML_FILE
		mv $GXD_XML_FILE.gz $GXD_LOG_DIR/xml/
	fi

done < $FILELIST

