#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON ControlledVocabulary XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnControlledVocabularyParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for cvo in self.xml_nodes(xml_file, "on/controlledVocabularies/controlledVocabulary"):

			cv = {}

			cvId 			= self.check( cvo, 'cvId' )
			updateId 		= self.check( cvo, 'updateId' )

			cv['cvId'] 		= cvId
			cv['updateId'] 	= updateId
			cv['updateDate']= self.check( cvo, 'updateDate' )
			cv['deleted'] 	= self.format_boolean( self.check( cvo, 'deleted' ) )
			cv['cvType']	= self.check( cvo, 'objectType' )

			cv['id'] 		= self.check( cvo.find('id') )
			cv['type'] 		= self.check( cvo.find('type') )
			cv['lang'] 		= self.check( cvo.find('lang') )
			cv['value'] 	= self.check( cvo.find('value') )
      
			if self.do_output_tsv:
				if not self.validate_and_write_tsv(cv, 'cv', 'cvId'):
					cvo.clear()
					continue

			for cvatt in cvo.findall('extendedAttributes/extendedAttribute'):

				att = {}

				att['cvId'] 			= cvId
				att['updateId'] 		= updateId

				att['attributeId'] 		= self.check( cvatt.find('attributeId') )
				att['attributeType'] 	= self.check( cvatt.find('attributeType') )
				att['attributeLang'] 	= self.check( cvatt.find('attributeLang') )
				att['attributeValue'] 	= self.check( cvatt.find('attributeValue') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(att, 'cv_attribute', 'cvId'):
						cvatt.clear()
						continue

				if self.do_output_jsonl:
					[ att.pop(key) for key in ['cvId','updateId'] ]
					cv['extendedAttributes'] = ( cv.get('extendedAttributes') or [] ) + [ att.copy() ]

				cvatt.clear()

			if self.do_output_jsonl:
				self.write_jsonl(cv, 'cv')

			cvo.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'ControlledVocabulary'

entities = [
	 'cv'
 ]

tables = [
	 'cv' 
	,'cv_attribute' 
]

fields = {
	 'cv': ['cvId', 'updateId', 'updateDate', 'deleted', 'cvType', 'id', 'type', 'lang', 'value', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'cv_attribute': ['cvId', 'updateId', 'attributeId', 'attributeType', 'attributeLang', 'attributeValue', 'MD5', 'ingestId']
}

# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnControlledVocabularyParser = GnControlledVocabularyParser( endpoint, entities, tables, fields )
gnControlledVocabularyParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnControlledVocabularyParser.run( xml_file )


