#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON ProgramMappings XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnProgramMappingsParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for m in self.xml_nodes(xml_file, "on/programMappings/programMapping"):

			mapping = {}
			mappingId 		= self.check( m, 'programMappingId' )
			updateId 		= self.check( m, 'updateId' )

			mapping['mappingId'] 	= mappingId
			mapping['updateId'] 	= updateId

			mapping['updateDate']	= self.check( m, 'updateDate' )
			mapping['status'] 		= self.check( m.find('status') )
			mapping['deleted'] 		= self.format_boolean( self.check( m, 'deleted' ) )

			mapping['creationDate'] = self.check( m.find('creationDate') )
			mapping['catalogName'] 	= self.check( m.find('catalogName') )
			mapping['avail_start'] 	= self.check( m.find('availability/start') )
			mapping['avail_end'] 	= self.check( m.find('availability/end') )
			mapping['message_reason'] = self.check( m.find('message'), 'reason' )
			mapping['message_detail'] = self.check( m.find('message/detail') )
      
      
			if self.do_output_tsv:
				if not self.validate_and_write_tsv(mapping, 'mapping', 'mappingId'):
					m.clear()
					continue


			for mi in m.findall('id'):

				mid = {}

				mid['mappingId'] 	= mappingId
				mid['updateId'] 	= updateId

				mid['id'] 			= self.check( mi )
				mid['type'] 		= self.check( mi, 'type' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(mid, 'mapping_id', 'mappingId'):
						mi.clear()
						continue

				if self.do_output_jsonl:
					[ mid.pop(key) for key in ['mappingId','updateId'] ]
					mapping['ids'] = ( mapping.get('ids') or [] ) + [ mid.copy() ]

				mi.clear()

			for ml in m.findall('link'):

				link = {}

				link['mappingId'] 	= mappingId
				link['updateId'] 	= updateId

				link['link'] 		= self.check( ml )
				link['idType'] 		= self.check( ml, 'idType' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(link, 'mapping_link', 'mappingId'):
						ml.clear()
						continue

				if self.do_output_jsonl:
					[ link.pop(key) for key in ['mappingId','updateId'] ]
					mapping['links'] = ( mapping.get('links') or [] ) + [ link.copy() ]

				ml.clear()

			if self.do_output_jsonl:
				self.write_jsonl(mapping, 'mapping')

			m.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'ProgramMappings'


entities = [
	 'mapping' 
]

tables = [
	 'mapping' 
	,'mapping_id' 
	,'mapping_link' 
]

fields = {
	 'mapping': ['mappingId', 'updateId', 'updateDate', 'deleted', 'creationDate', 'status', 'catalogName', 'avail_start', 'avail_end', 'message_reason', 'message_detail', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'mapping_id': ['mappingId', 'updateId', 'id', 'type', 'MD5', 'ingestId']
	,'mapping_link': ['mappingId', 'updateId', 'link', 'idType', 'MD5', 'ingestId']
}

# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnProgramMappingsParser = GnProgramMappingsParser( endpoint, entities, tables, fields )
gnProgramMappingsParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnProgramMappingsParser.run( xml_file )


