#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON Sports XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnSportsParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for s in self.xml_nodes(xml_file, "on/sports/sport"):

			sport = {}

			sportGId 					= self.check( s, 'sportGId')
			updateId 					= self.check( s, 'updateId' )
			updateDate 					= self.check( s, 'updateDate' )
			is_deleted					= self.format_boolean( self.check( s, 'deleted' ) )

			sport['sportGId'] 			= sportGId
			sport['updateId'] 			= updateId
			sport['updateDate']			= updateDate
			sport['deleted'] 			= is_deleted

			name_default = None
			for n in s.findall('names/name'):
				if 'default' == self.check( n, 'type'):
					name_default = self.check( n )

			sport['name_default']	= name_default

			if self.do_output_tsv:
				if not self.validate_and_write_tsv(sport, 'sport', 'sportGId'):
					s.clear()
					continue

			for sn in s.findall('names/name'):

				name = {}

				name['sportGId']	= sportGId
				name['updateId'] 	= updateId
				name['name'] 		= self.check( sn )
				name['type'] 		= self.check( sn, 'type' )
				name['lang'] 		= self.check( sn, 'lang' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(name, 'sport_name', 'sportGId'):
						sn.clear()
						continue

				if self.do_output_jsonl:
					[ name.pop(key) for key in ['sportGId','updateId'] ]
					sport['names'] = ( sport.get('names') or [] ) + [ name.copy() ]

				sn.clear()

			if self.do_output_jsonl:
				self.write_jsonl(sport, 'sport')

			s.clear()


# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'Sports'


entities = [
	 'sport' 
]

tables = [
	 'sport' 
	,'sport_name' 
]

fields = {
	 'sport': ['sportGId', 'updateId', 'updateDate', 'deleted', 'name_default', 'MD5', 'ingestId']
	,'sport_name': ['sportGId', 'updateId', 'name', 'type', 'lang', 'MD5', 'ingestId']

}


# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnSportsParser = GnSportsParser( endpoint, entities, tables, fields )
gnSportsParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnSportsParser.run( xml_file )
