#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Parses a given ON Teams XML file
#  Validates each record against fields configuration file
#  Writes validated records into TSV files for the endpoint
#  =============================================================================

import os
import sys

sys.path.append(os.path.join(sys.path[0],'..'))
import gxd_functions as gxd

xml_file 	= sys.argv[1]
files_path 	= sys.argv[2]

class GnTeamsParser(gxd.GnEndpointParser):

	def parse(self, xml_file):

		for t in self.xml_nodes(xml_file, "on/teams/team"):

			team = {}

			teamId 					= self.check( t, 'teamId')
			updateId 				= self.check( t, 'updateId' )
			updateDate 				= self.check( t, 'updateDate' )
			is_deleted 				= self.format_boolean( self.check( t, 'deleted' ) )

			team['teamId'] 			= teamId
			team['updateId'] 		= updateId
			team['updateDate']		= updateDate
			team['deleted'] 		= is_deleted

			team['teamGId'] 		= self.check( t, 'teamGId' )
			team['sportGId'] 		= self.check( t, 'sportGId' )
			team['type'] 			= self.check( t.find('type') )
			team['class'] 			= self.check( t.find('class') )
			team['venueGId'] 		= self.check( t.find('venueGId') )
			team['universityGId'] 	= self.check( t.find('universityGId') )

			team['gender'] 			= self.check( t.find('gender') )
			team['country'] 		= self.check( t.find('country') )
			team['genreId'] 		= self.check( t.find('sport'), 'genreId' )
			team['venueId'] 		= self.check( t.find('venueId') )
			team['universityId'] 	= self.check( t.find('universityId') )

			team['address_city'] 		= self.check( t.find('address/city') )
			team['address_state'] 		= self.check( t.find('address/state') )
			team['address_country'] 	= self.check( t.find('address/country') )

			for tc in t.findall('colors/color'):
				if self.check( tc, 'type' ) == 'primary':
					team['color_primary'] = self.check( tc )
				elif self.check( tc, 'type' ) == 'secondary':
					team['color_secondary'] = self.check( tc )


			if self.do_output_tsv:
				if not self.validate_and_write_tsv(team, 'team', 'teamId'):
					t.clear()
					continue

			for tb in t.findall('teamBrands/teamBrand'):

				teambrand = {}

				teamBrandId 				= self.check( tb.find('teamBrandId') )

				teambrand['teamBrandId'] 	= teamBrandId
				teambrand['updateId'] 		= updateId
				teambrand['updateDate']		= updateDate
				teambrand['teamId'] 		= teamId
				teambrand['deleted'] 		= is_deleted

				teambrand['teamBrandGId'] 	= self.check( tb.find('teamBrandGId') )

				teambrand['active']			= self.format_boolean( self.check( tb.find('active') ) )

				teambrand['fromDate'] 		= self.format_datetime( self.check( tb.find('fromDate') ), '%Y-%m-%d')
				teambrand['toDate'] 		= self.format_datetime( self.check( tb.find('toDate')   ), '%Y-%m-%d')
				teambrand['names_lang'] 	= self.check( tb.find('names'), 'lang' )

				teambrand['officialName'] 	= self.check( tb.find('officialName') )
				teambrand['properName'] 	= self.check( tb.find('properName') )
				teambrand['nickName'] 		= self.check( tb.find('nickName') )
				teambrand['abbreviation'] 	= self.check( tb.find('abbreviation') )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(teambrand, 'teambrand', 'teamBrandId'):
						tb.clear()
						continue


				for tbn in tb.findall('names/name'):

					name = {}

					name['teamBrandId']	= teamBrandId
					name['updateId'] 		= updateId
					name['name'] 			= self.check( tbn )
					name['type'] 			= self.check( tbn, 'type' )
					name['lang'] 			= self.check( tbn, 'lang' )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(name, 'teambrand_name', 'teamBrandId'):
							tbn.clear()
							continue

					if self.do_output_jsonl:
						[ name.pop(key) for key in ['teamBrandId','updateId'] ]
						teambrand['names'] = ( teambrand.get('names') or [] ) + [ name.copy() ]

					tbn.clear()

				for tba in tb.findall('assets/asset'):

					asset = {}

					asset['teamBrandId']	= teamBrandId
					asset['updateId'] 		= updateId
					asset['assetId'] 		= self.check( tba, 'assetId' )
					asset['assetUpdateId'] 	= self.check( tba, 'updateId' )
					asset['lastModified'] 	= self.format_datetime( self.check( tba, 'lastModified' ))
					asset['expiredDate'] 	= self.check( tba, 'expiredDate' )
					asset['type'] 			= self.check( tba, 'type' )
					asset['width'] 			= self.check( tba, 'width' )
					asset['height'] 		= self.check( tba, 'height' )
					asset['ratio'] 			= self.check( tba, 'ratio' )
					asset['tier']	 		= self.check( tba, 'tier' )
					asset['category'] 		= self.check( tba, 'category' )
					asset['deleted'] 		= self.format_boolean( self.check( tba, 'deleted' ))
					asset['primary'] 		= self.format_boolean( self.check( tba, 'primary' ))
					asset['URI'] 			= self.check( tba.find('URI') )

					if self.do_output_tsv:
						if not self.validate_and_write_tsv(asset, 'teambrand_asset', 'teamBrandId'):
							tba.clear()
							continue

					if self.do_output_jsonl:
						[ asset.pop(key) for key in ['teamBrandId','updateId'] ]
						teambrand['assets'] = ( teambrand.get('assets') or [] ) + [ asset.copy() ]

					tba.clear()

				if self.do_output_jsonl:
					self.write_jsonl(teambrand, 'teambrand')

				tb.clear()

			for tg in t.findall('genres/genre'):

				genre = {}

				genre['teamId'] 		= teamId
				genre['updateId'] 		= updateId

				genre['genre']			= self.check( tg )
				genre['genreId'] 		= self.check( tg, 'genreId' )

				if self.do_output_tsv:
					if not self.validate_and_write_tsv(genre, 'team_genre', 'teamId'):
						tg.clear()
						continue

				if self.do_output_jsonl:
					[ genre.pop(key) for key in ['teamId','updateId'] ]
					team['genres'] = ( team.get('genres') or [] ) + [ genre.copy() ]

				tg.clear()

			if self.do_output_jsonl:
				self.write_jsonl(team, 'team')

			t.clear()



# ================== ENDPOINT SPECIFIC INIT CODE BEGIN ==================
# Here insert the endpoint-specific code from schema/release/<product>/parse_.py

endpoint = 'Teams'

entities = [
	 'team' 
	,'teambrand' 
]

tables = [
	 'team' 
	,'teambrand' 
	,'team_genre'
	,'teambrand_name'
	,'teambrand_asset' 
]

fields = {
	 'team': ['teamId', 'updateId', 'updateDate', 'deleted', 'teamGId', 'type', 'class', 'color_primary', 'color_secondary', 'gender', 'country', 'genreId', 'sportGId', 'venueId', 'venueGId', 'universityId', 'universityGId', 'address_city', 'address_state', 'address_country', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'teambrand': ['teamBrandId', 'updateId', 'updateDate', 'deleted', 'teamBrandGId', 'teamId', 'active', 'fromDate', 'toDate', 'names_lang', 'officialName', 'properName', 'nickName', 'abbreviation', 'MD5', 'ingestId', 'batchUpdateOrd']
	,'team_genre': ['teamId', 'updateId', 'genre', 'genreId', 'MD5', 'ingestId']
	,'teambrand_name': ['teamBrandId', 'updateId', 'name', 'type', 'lang', 'MD5', 'ingestId']
	,'teambrand_asset': ['teamBrandId', 'updateId', 'assetUpdateId', 'assetId', 'deleted', 'lastModified', 'expiredDate', 'type', 'width', 'height', 'primary', 'category', 'ratio', 'tier', 'URI', 'MD5', 'ingestId']
}

# =================== ENDPOINT SPECIFIC INIT CODE END ===================


gnTeamsParser = GnTeamsParser( endpoint, entities, tables, fields )
gnTeamsParser.set_config( os.environ['GXD_PDK_DIR'], os.environ['GXD_PDK_CONFIG'], files_path )
gnTeamsParser.run( xml_file )
