-- {0}: core schema
-- {1}: pg username


CREATE TABLE IF NOT EXISTS {0}._stats (
    "timestamp" timestamp without time zone NOT NULL,
    "metric" character varying(255) NOT NULL,
    "dimension" character varying(255),
    "value" real );

CREATE INDEX IF NOT EXISTS "_stats_timestamp" ON {0}._stats USING btree ("timestamp");
CREATE INDEX IF NOT EXISTS "_stats_metric" ON {0}._stats USING btree ("metric");


-- update_date stats for source
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP(CEILING((EXTRACT('epoch' FROM t."updateDate") / 900 )) * 900) AT TIME ZONE 'UTC' AS "timestamp"
    ,'source_updates' AS metric, COUNT(t."prgSvcId") AS value
FROM {0}.source t
WHERE t."updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'source_updates')
GROUP BY "timestamp" ORDER BY "timestamp" DESC;

-- update_date stats for lineup
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP(CEILING((EXTRACT('epoch' FROM t."updateDate") / 900 )) * 900) AT TIME ZONE 'UTC' AS "timestamp"
    ,'lineup_updates' AS metric, COUNT(t."lineupId") AS value
FROM {0}.lineup t
WHERE t."updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'lineup_updates')
GROUP BY "timestamp" ORDER BY "timestamp" DESC;

-- update_date stats for celebrity
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP(CEILING((EXTRACT('epoch' FROM t."updateDate") / 900 )) * 900) AT TIME ZONE 'UTC' AS "timestamp"
    ,'celebrity_updates' AS metric, COUNT(t."personId") AS value
FROM {0}.celebrity t
WHERE t."updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'celebrity_updates')
GROUP BY "timestamp" ORDER BY "timestamp" DESC;

-- update_date stats for program
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP(CEILING((EXTRACT('epoch' FROM t."updateDate") / 900 )) * 900) AT TIME ZONE 'UTC' AS "timestamp"
    ,'program_updates' AS metric, COUNT(t."TMSId") AS value
FROM {0}.program t
WHERE t."updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'program_updates')
GROUP BY "timestamp" ORDER BY "timestamp" DESC;

-- update_date stats for availability
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP(CEILING((EXTRACT('epoch' FROM t."updateDate") / 900 )) * 900) AT TIME ZONE 'UTC' AS "timestamp"
    ,'availability_updates' AS metric, COUNT(t."availabilityId") AS value
FROM {0}.availability t
WHERE t."updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'availability_updates')
GROUP BY "timestamp" ORDER BY "timestamp" DESC;

-- update_date stats for mapping
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP(CEILING((EXTRACT('epoch' FROM t."updateDate") / 900 )) * 900) AT TIME ZONE 'UTC' AS "timestamp"
    ,'mapping_updates' AS metric, COUNT(t."mappingId") AS value
FROM {0}.mapping t
WHERE t."updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'mapping_updates')
GROUP BY "timestamp" ORDER BY "timestamp" DESC;

-- update_date stats for schedule
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP(CEILING((EXTRACT('epoch' FROM t."updateDate") / 900 )) * 900) AT TIME ZONE 'UTC' AS "timestamp"
    ,'schedule_updates' AS metric, COUNT(t."scheduleId") AS value
FROM {0}.schedule t
WHERE t."updateDate" > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'schedule_updates')
GROUP BY "timestamp" ORDER BY "timestamp" DESC;


-- ingest_id stats for source
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) AS "timestamp",
    'source_ingests' AS metric, COUNT(t."prgSvcId") AS value
FROM {0}.source t
WHERE TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'source_ingests')
GROUP BY t."ingestId" ORDER BY t."ingestId" DESC;

-- ingest_id stats for lineup
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) AS "timestamp",
    'lineup_ingests' AS metric, COUNT(t."lineupId") AS value
FROM {0}.lineup t
WHERE TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'lineup_ingests')
GROUP BY t."ingestId" ORDER BY t."ingestId" DESC;

-- ingest_id stats for celebrity
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) AS "timestamp",
    'celebrity_ingests' AS metric, COUNT(t."personId") AS value
FROM {0}.celebrity t
WHERE TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'celebrity_ingests')
GROUP BY t."ingestId" ORDER BY t."ingestId" DESC;

-- ingest_id stats for program
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) AS "timestamp",
    'program_ingests' AS metric, COUNT(t."TMSId") AS value
FROM {0}.program t
WHERE TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'program_ingests')
GROUP BY t."ingestId" ORDER BY t."ingestId" DESC;

-- ingest_id stats for availability
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) AS "timestamp",
    'availability_ingests' AS metric, COUNT(t."availabilityId") AS value
FROM {0}.availability t
WHERE TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'availability_ingests')
GROUP BY t."ingestId" ORDER BY t."ingestId" DESC;

-- ingest_id stats for mapping
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) AS "timestamp",
    'mapping_ingests' AS metric, COUNT(t."mappingId") AS value
FROM {0}.mapping t
WHERE TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'mapping_ingests')
GROUP BY t."ingestId" ORDER BY t."ingestId" DESC;

-- ingest_id stats for schedule
INSERT INTO {0}._stats ("timestamp", "metric", "value")
SELECT TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) AS "timestamp",
    'schedule_ingests' AS metric, COUNT(t."scheduleId") AS value
FROM {0}.schedule t
WHERE TO_TIMESTAMP( t."ingestId"::VARCHAR(14), 'YYYYMMDDHH24MISS' ) > (SELECT COALESCE(MAX("timestamp"),'1970-01-01') FROM {0}._stats WHERE metric = 'schedule_ingests')
GROUP BY t."ingestId" ORDER BY t."ingestId" DESC;
