-- Extract the schedule of a channel along with basic program information in a table with each airing being one row.
-- Include source, schedule, schedule event, broadcaster ids and basic program information
-- Using "RTL Zwei" 83642 as example
SELECT
	 s."prgSvcId", s."name", s."callSign"
	,DATE(h."date") as "date"
	,he.time AS "at", he."dur", he."TMSId"
  	,p."progType", p."subType"
  	,pt."title"
  	,pei."title" AS ep_title, pei."number", pei."season", pei."seasonYear"
	,he."quals",he."tvRating" AS "scheduleRating"
	,(
		SELECT GROUP_CONCAT(g."genre")
		FROM program_genre g
		WHERE 1=1
			AND g."TMSId" = p."TMSId"
	) AS "genres"
	,(
		SELECT GROUP_CONCAT(hb."broadcastId_type" || ':' || hb."broadcastId")
		FROM schedule_event_bcastid hb
		WHERE 1=1
			AND hb."eventId" = he."eventId"
	) AS "broadcasterIds"
	,(
		SELECT GROUP_CONCAT(pr."ratingsBody" || ':' || pr."rating_code")
		FROM program_rating pr
		WHERE 1=1
			AND pr."TMSId" = p."TMSId"
			-- FSK and FSF only
			AND pr."ratingBodyId" IN (22, 24)
	) AS "programRating"
FROM
	schedule h 
	JOIN schedule_event he ON h."scheduleId" = he."scheduleId"
	JOIN source s ON s."prgSvcId" = h."prgSvcId"
	JOIN program p ON he."TMSId" = p."TMSId"
	LEFT JOIN program_title pt ON p."TMSId" = pt."TMSId" AND pt."type"='full'
	LEFT JOIN program_epinfo pei ON p."TMSId" = pei."TMSId"
WHERE 1=1
	AND h."prgSvcId" IN (83642)
	AND DATE(h."date") BETWEEN DATE('now','-1 day') AND DATE('now','+14 day')
ORDER BY 1,4,5
;
