import sys
import csv
import collections

dl_log = sys.argv[1]
pl_sizes = sys.argv[2]
out_file = sys.argv[3]
location = sys.argv[4]

payload_size = {}
with open(pl_sizes) as f:
	for row in csv.reader(f):
		ingest_id = row[0].split("/")[2]
		value = row[1]
		key = ingest_id + ":" + row[2]
		payload_size[key] = value

log = []
with open(dl_log) as f:
	rows = csv.reader(f)
	for row in rows:

		entry = collections.OrderedDict()
		entry['location'] = location

		try:
			ingest_id = row[0].split("/")[2]
			timestamp = int(row[0].split(":")[1])
			url = row[1]

			entry['ingest_id'] = ingest_id
			entry['ord'] = 1
			entry['timestamp'] = timestamp
			
			entry['base'] = url.split("?")[0].split("/")[2]
			entry['endpoint'] = url.split("?")[0].split("/")[-1]

			params = url.split("?")[1].split("&")
			entry['update_id'] = params[0].split("=")[1]
			entry['limit'] = params[1].split("=")[1]
			entry['api_key'] = params[2].split("=")[1]
			if len(params) > 3:
				entry['stream_id'] = params[3].split("=")[1]
			else:
				entry['stream_id'] = None

		except IndexError as e:
			print("Exception on line {0} of the download log: {1}".format(rows.line_num, e))
			sys.exit(1)

		# Skip entry if it has no payload size
		key = ingest_id + ":" + row[2]
		if payload_size.get(key):
			entry['payload_size'] = int(payload_size.get(key))
		else:
			continue

		# If ingest_id is the same as in last entry, calculate duration and bps, update current ord
		# Else, check if last entry is the only one for that ingest_id (ord==1) and remove it if so
		if len(log) > 0:
			prev_entry = log[-1]
			if entry['ingest_id'] == prev_entry['ingest_id']:
				entry['ord'] = prev_entry['ord'] + 1

				# Calculate current entry duration
				prev_entry['duration'] = timestamp - prev_entry['timestamp']

				# Calculate current entry bps (round up seconds to 1 to avoid division by zero)
				if prev_entry['duration'] == 0:
					prev_entry['duration'] = 1
				prev_entry['bytes_per_sec'] = prev_entry['payload_size'] // prev_entry['duration']

			#elif prev_entry['ord'] == 1:
			else:
				log.pop()

		log.append(entry)

# Remove the last entry if it is the only entry for that ingest_id
if len(log) > 0 and log[-1]['ord'] == 1:
	log.pop()

# Output to CSV
if len(log) > 0:
	header = log[0].keys()
	with open(out_file + ".csv", 'w', newline='')  as out:
		dict_writer = csv.DictWriter(out, header)
		dict_writer.writeheader()
		dict_writer.writerows(log)
else:
	print("Nothing to write")

