#  -----------------------------------------------------------------------------
#  Gracenote (A Nielsen Company) Proprietary Information
#  This software is supplied under the terms of a license agreement or
#  nondisclosure agreement with Nielsen/Gracenote, Inc. and may not be copied
#  or disclosed except in accordance with the terms of that agreement.
#  Copyright(c) 2020-present. Gracenote, A Nielsen Company, All Rights Reserved.
#  -----------------------------------------------------------------------------

#  =============================================================================
#  Remove/drop all state variables, temp data and configured DB schemas
#  =============================================================================

while getopts 'c:' OPTION; do
	case $OPTION in
		c) CONFIG=$OPTARG ;;
	esac
done

if [ -z "$CONFIG" ]; then
	echo "USAGE: $0 -c my.conf"
	echo
fi

# The script can be invoked either from PDK root dir or from utils dir
if [ -e ./.utils ]; then
	DIR=..
else
	DIR=.
fi

export GXD_PDK_DIR=$DIR
export GXD_PDK_CONFIG=$CONFIG
export GXD_LOG=/dev/null

. $DIR/bin/functions.sh
load_config

echo
echo "WARNING! All current variables and any existing files in files/xml or files/tsv will be deleted!" 
echo
echo "Are you sure you want to proceed (y/n)? "
read ANSWER
if [ "$ANSWER" = "${ANSWER#[Yy]}" ]; then
    exit 
fi

echo
echo Removing all variables..
$DIR/utils/rm_vars.sh
echo Removing any remaining XML files..
$DIR/utils/rm_xml.sh
echo Removing any remaining TSV files..
$DIR/utils/rm_tsv.sh
echo Removing any remaining JSONL files..
$DIR/utils/rm_jsonl.sh

echo Removing DB data..
if [ $DATABASE = "SQLITE" ]; then
	$DIR/utils/rm_schema.sqlite.sh -c $CONFIG
elif [ $DATABASE = "PGSQL" ]; then
	$DIR/utils/rm_schema.pgsql.sh -c $CONFIG
elif [ $DATABASE = "MYSQL" ]; then
	$DIR/utils/rm_schema.mysql.sh -c $CONFIG
fi
